package main;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import function.FunctionFactory;
import function.IFunction;
import operand.IOperand;
import operand.OperandFactory;
import operator.IOperator;
import operator.OperatorFactory;
import simplifyRule.ISimplifyRule;
import simplifyRule.SimplifyRuleFactory;

public class MyFileReader {

	public MyFileReader() {
	}

	public ISimplify getSimplify() {
		return getSimplify("main.NSimplify");
	}

	public ISimplify getSimplify(String simplifyName) {

		ISimplify simplify = SimplifyFactory.createSimplify(simplifyName);

		HashMap<String, IOperand> operands = getOperandInfo();
		simplify.setEquivalanceCheckNodes(getEquivalanceCheckNodes(operands));
		simplify.setOperand(operands);
		simplify.setOperator(getOperatorInfo());

		return simplify;
	}

	/**
	 * setting operand info
	 * 
	 * @return
	 */
	public HashMap<String, IOperand> getOperandInfo() {

		HashMap<String, IOperand> operands = new HashMap<String, IOperand>();

		try {
			BufferedReader reader = new BufferedReader(new FileReader(
					"operand.csv"));

			while (reader.ready()) {
				String line = reader.readLine();
				if (!isSkip(line)) {
					String[] info = line.split(",");

					if (info.length != 2) {
						System.out.println("Operand info is wrong.");
						System.exit(-1);
					}

					IOperand temp = OperandFactory
							.createOperand("operand.NOperand");

					boolean valuable;
					double value;
					if (info[1].equals("val")) {
						value = Double.NaN;
						valuable = true;
					} else if (info[1].equals("Math.PI")) {
						value = Math.PI;
						valuable = false;
					} else if (info[1].equals("Math.E")) {
						value = Math.E;
						valuable = false;
					} else {
						value = Double.parseDouble(info[1]);
						valuable = false;
					}

					temp.setAllParams(info[0], value, valuable);
					operands.put(info[0], temp);
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
			System.exit(-1);
		}

		return operands;
	}

	/**
	 * setting operator info
	 * 
	 * @return
	 */
	public HashMap<String, IOperator> getOperatorInfo() {

		HashMap<String, IOperator> operators = new HashMap<String, IOperator>();

		try {
			BufferedReader reader = new BufferedReader(new FileReader(
					"operator.csv"));

			while (reader.ready()) {
				String line = reader.readLine();
				if (!isSkip(line)) {
					String[] info = line.split(",");

					if (info.length != 5) {
						System.out.println("Operator info is wrong.");
						System.exit(-1);
					}

					IOperator temp = OperatorFactory
							.createOperator("operator.NOperator");

					IFunction function = FunctionFactory
							.createFunction(info[1]);
					ISimplifyRule simplifyRule = SimplifyRuleFactory
							.create(info[3]);
					boolean changeable = true;
					if (info[4].equals("commutative")) {
						changeable = true;
					} else if (info[4].equals("noncommutative")) {
						changeable = false;
					} else {
						System.out
								.println("operator " + info[0] + " is wrong.");
					}

					temp.setAllParams(info[0], function, Integer
							.parseInt(info[2]), simplifyRule, changeable);

					operators.put(info[0], temp);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			System.exit(-1);
		}

		return operators;
	}

	/**
	 * setting operand using equivalent based simplification
	 * 
	 * @return
	 */
	public HashSet<String> getEquivalanceCheckNodes(
			HashMap<String, IOperand> operands) {

		HashSet<String> nodes = new HashSet<String>();

		try {
			BufferedReader reader = new BufferedReader(new FileReader(
					"equivalanceNode.csv"));

			while (true) {
				String line = reader.readLine();
				if (!isSkip(line)) {
					String[] temp = line.split(",");

					if (temp.length != 2) {
						System.out.println("Zero or relative error is wrong.");
						System.exit(-1);
					}

					zeroAndRelativeError.setZero(Double.parseDouble(temp[0]));
					zeroAndRelativeError.setRelativeError(Double
							.parseDouble(temp[1]));
					break;

				}
			}

			while (reader.ready()) {
				String line = reader.readLine();
				if (!isSkip(line)) {
					String[] temp = line.split(",");

					if (temp.length != 2) {
						System.out.println("Equivalent node info is wrong.");
						System.exit(-1);
					}

					if (!operands.containsKey(temp[0])) {
						IOperand operand = OperandFactory
								.createOperand("operand.NOperand");
						boolean valuable;
						double value;
						if (temp[1].equals("val")) {
							valuable = true;
							value = Double.NaN;
						} else {
							valuable = false;
							value = Double.parseDouble(temp[1]);
						}
						operand.setAllParams(temp[0], value, valuable);
						operands.put(temp[0], operand);
					}

					nodes.add(temp[0]);
				}
			}

		} catch (Exception e) {
			e.printStackTrace();
			System.exit(-1);
		}

		return nodes;
	}

	/**
	 * setting test point
	 * 
	 * @param line
	 * @return
	 */
	public HashMap<String, double[]> getTestPoint(String line,
			HashMap<String, IOperand> operand) {

		HashMap<String, double[]> testPoints = new HashMap<String, double[]>();

		try {

			String[] temp = line.split(":");
			for (int i = 0; i < temp.length; i++) {

				String[] temp2 = temp[i].split(",");
				String symbol = temp2[0];
				double[] value = new double[temp2.length - 1];
				for (int j = 1; j < temp2.length; j++) {
					value[j - 1] = Double.parseDouble(temp2[j]);
				}
				testPoints.put(symbol, value);
			}

		} catch (NumberFormatException e) {
			e.printStackTrace();
			System.exit(-1);
		}

		Iterator<String> it = operand.keySet().iterator();
		while (it.hasNext()) {
			String symbol = it.next();
			if (operand.get(symbol).isValuable()
					&& !testPoints.containsKey(symbol)) {
				System.out.println("no test point oprand " + symbol);
				System.exit(-1);
			}
		}

		return testPoints;
	}

	public boolean isSkip(String line) {
		if (line.compareTo("") == 0 || line.charAt(0) == '#') {
			return true;
		}
		return false;
	}

	/**
	 * example
	 * 
	 * @param args
	 */
	public static void main(String[] args) {

		MyFileReader fileReader = new MyFileReader();
		ISimplify simplify = fileReader.getSimplify();

		zeroAndRelativeError.setZero(1E-10);
		zeroAndRelativeError.setRelativeError(1E-3);
		System.out.println(zeroAndRelativeError.getZero());
		System.out.println(zeroAndRelativeError.getRelativeError());

		HashMap<String, IOperand> operands = simplify.getOperand();
		System.out.println(operands.get("X").isValuable());
		System.out.println(operands.get("0").getValue());

		HashMap<String, IOperator> operators = simplify.getOperator();
		System.out.println(operators.get("S").getSymbol());
		System.out.println(operators.get("*").getFunction().calculate(2, 4));

		System.out.println(simplify.getEquivalanceCheckNodes());

		String str = "X,1,3,5,7:Y,2,4,6,8:Z,3,6,9,12";
		HashMap<String, double[]> testPoint = fileReader.getTestPoint(str,
				operands);
		double[] temp = testPoint.get("X");
		System.out.println(temp[0]);
		temp = testPoint.get("Y");
		System.out.println(temp[1]);
		temp = testPoint.get("Z");
		System.out.println(temp[3]);
	}
}
