package main;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import operand.IOperand;
import operator.IOperator;

public class NSimplify implements ISimplify {

	// test point
	private HashMap<String, double[]> testPoints_;

	// operator
	private HashMap<String, IOperator> operators_ = new HashMap<String, IOperator>();

	// operand
	private HashMap<String, IOperand> operands_ = new HashMap<String, IOperand>();

	// nodes using equivalent based simplification
	private HashSet<String> equivalanceCheckNodes_ = new HashSet<String>();

	// 
	private NDecimialNumberArray nArray_ = new NDecimialNumberArray();

	/**
	 * defult constructor
	 * 
	 */
	public NSimplify() {
	}

	/**
	 * rule and equivalent simplification
	 */
	public String ruleAndEquivalanceSimplify(String target) {

		String old, result = target;
		do {
			old = result;
			result = equivalanceSimplify(ruleSimplify(result));
		} while (!result.equals(old));
		return result;
	}

	/**
	 * rule simplification
	 */
	public String ruleSimplify(String target) {

		if (target.length() == 1) {
			return target;
		}

		String result = target;
		MyTreeNode node = MyTreeUtil.postOrderToTree(target);
		boolean isChange = true;

		while (true) {

			if (!isChange || result.length() == 1) {
				break;
			}

			isChange = false;
			boolean frag = false;

			Enumeration em = node.breadthFirstEnumeration();

			int currentLevel = 0;

			while (em.hasMoreElements()) {

				MyTreeNode element = (MyTreeNode) em.nextElement();

				if (isChange && element.getLevel() > currentLevel) {
					break;
				}

				currentLevel = element.getLevel();

				String str = element.toString();

				if (operators_.containsKey(str)) {
					frag = operators_.get(str).getSimplifyRule().simplify(
							element);

					if (!isChange && frag) {
						isChange = true;
					}
				}
			}

		}
		result = MyTreeUtil.TreeToPostOrder(node);
		return result;
	}

	/**
	 * equivalent simplification
	 */
	public String equivalanceSimplify(String target) {

		MyTreeNode targetNode = MyTreeUtil.postOrderToTree(target);

		Enumeration en = targetNode.breadthFirstEnumeration();
		while (en.hasMoreElements()) {

			HashSet<String> isEquivalance = new HashSet<String>(equivalanceCheckNodes_);
			MyTreeNode node = (MyTreeNode) en.nextElement();

			do {
				double nodeValue = getFunctionValue(node);

				Iterator<String> it2 = equivalanceCheckNodes_.iterator();
				while (it2.hasNext()) {
					String symbol = it2.next();
					if (testPoints_.containsKey(symbol)) {
						double val = testPoints_.get(symbol)[nArray_
								.getNumOfSymbol(symbol)];
						if (!isSameValue(nodeValue, val)) {
							isEquivalance.remove(symbol);
						}

					} else {
						if (!isSameValue(nodeValue, operands_.get(symbol)
								.getValue())) {
							isEquivalance.remove(symbol);
						}
					}
				}
				if (isEquivalance.isEmpty()) {
					break;
				}
			} while (!nArray_.next());
			nArray_.reset();

			if (isEquivalance.size() == 1) {
				Iterator<String> it2 = isEquivalance.iterator();
				while (it2.hasNext()) {
					String symbol = it2.next();
					node.setUserObject(symbol);
					node.removeAllChildren();
				}
			}

		}

		return MyTreeUtil.TreeToPostOrder(targetNode);
	}

	/**
	 * check node value
	 * 
	 * @param target
	 * @return
	 */
	public double getFunctionValue(MyTreeNode target) {

		String symbol = target.toString();

		if (operands_.containsKey(symbol)) {
			if (testPoints_.containsKey(symbol)) {
				return testPoints_.get(symbol)[nArray_.getNumOfSymbol(symbol)];
			} else {
				return operands_.get(symbol).getValue();
			}
		} else if (operators_.containsKey(symbol)) {

			int numOfOperand = operators_.get(symbol).numOfOperand();
			double[] childValue = new double[numOfOperand];

			for (int i = 0; i < numOfOperand; i++) {
				childValue[i] = getFunctionValue((MyTreeNode) target
						.getChildAt(i));
			}

			return operators_.get(symbol).getFunction().calculate(childValue);

		} else {
			System.out.println("Operand|Operator " + symbol + " is wrong.");
			System.exit(-1);
			return Double.NaN;
		}
	}

	/**
	 * equal check
	 */
	public boolean isSameValue(double value1, double value2) {

		if (isZero(value1) && isZero(value2)) {
			return true;
		} else if (isZero(value1) && !isZero(value2)) {
			return false;
		}
		return Math.abs((value1 - value2) / value1) < zeroAndRelativeError
				.getRelativeError();
	}

	/**
	 * zero check
	 */
	public boolean isZero(double value) {
		if (Math.abs(value) < zeroAndRelativeError.getZero()) {
			return true;
		} else {
			return false;
		}
	}

	public HashMap<String, IOperand> getOperand() {
		return operands_;
	}

	public void setOperand(HashMap<String, IOperand> operands) {
		operands_ = operands;
	}

	public HashMap<String, IOperator> getOperator() {
		return operators_;
	}

	public void setOperator(HashMap<String, IOperator> operators) {
		operators_ = operators;
	}

	public HashSet<String> getEquivalanceCheckNodes() {
		return equivalanceCheckNodes_;
	}

	public void setEquivalanceCheckNodes(HashSet<String> equivalanceCheckNodes) {
		equivalanceCheckNodes_ = equivalanceCheckNodes;
	}

	public HashMap<String, double[]> getTestPoint() {
		return testPoints_;
	}

	public void setTestPoint(HashMap<String, double[]> testPoints) {
		testPoints_ = testPoints;
		ArrayList<NDecimialNumber> points = new ArrayList<NDecimialNumber>();
		Iterator<String> it = testPoints_.keySet().iterator();
		while (it.hasNext()) {
			String symbol = it.next();
			int length = testPoints_.get(symbol).length;
			NDecimialNumber point = new NDecimialNumber(length, symbol);
			points.add(point);
		}
		nArray_.setArray(points);
	}

	public NDecimialNumberArray getNArray() {
		return nArray_;
	}

	public void setNArray(NDecimialNumberArray nArray) {
		nArray_ = nArray;
	}
}
