package test;

import java.util.HashMap;
import java.util.HashSet;

import main.MyFileReader;
import main.MyTreeNode;
import main.MyTreeUtil;
import main.zeroAndRelativeError;

import function.IFunction;
import operand.IOperand;
import operator.IOperator;
import simplifyRule.ISimplifyRule;
import junit.framework.TestCase;

public class MyFileReaderTest extends TestCase {

	MyFileReader reader_ = new MyFileReader();

	public void testSetOperandInfo() {
		HashMap<String, IOperand> operands = reader_.getOperandInfo();

		IOperand temp = operands.get("X");
		assertEquals("X", temp.getSymbol());
		assertEquals(true, temp.isValuable());

		temp = operands.get("1");
		assertEquals("1", temp.getSymbol());
		assertEquals(false, temp.isValuable());
		assertEquals(1.0, temp.getValue());

		// temp = operands.get("0");
		// assertEquals("0", temp.getSymbol());
		// assertEquals(false, temp.isValuable());
		// assertEquals(0.0, temp.getValue());
	}

	public void testSetOperatorInfo() {
		HashMap<String, IOperator> operator = reader_.getOperatorInfo();

		IOperator temp = operator.get("+");
		assertEquals("+", temp.getSymbol());

		ISimplifyRule rule = temp.getSimplifyRule();
		MyTreeUtil.setSimplify(reader_.getSimplify());
		MyTreeNode element = MyTreeUtil.postOrderToTree("1 0 +");
		assertEquals(true, rule.simplify(element));

		IFunction function = temp.getFunction();
		assertEquals(3.3, function.calculate(1.1, 2.2), 1E-4);

		boolean isChangeable = operator.get("+").isChangeable();
		assertTrue(isChangeable);
		isChangeable = operator.get("S").isChangeable();
		assertTrue(!isChangeable);
	}

	public void testSetEquivalanceCheckNodes() {
		HashMap<String, IOperand> operands = reader_.getOperandInfo();
		HashSet<String> equivalanceNode = reader_
				.getEquivalanceCheckNodes(operands);
		assertTrue(equivalanceNode.contains("X"));
		assertTrue(equivalanceNode.contains("0"));
		assertTrue(equivalanceNode.contains("1"));
		assertTrue(!equivalanceNode.contains("abc"));

		assertEquals(1E-9, zeroAndRelativeError.getZero());
		assertEquals(1E-4, zeroAndRelativeError.getRelativeError());
	}

	public void testSetTestPoint() {
		HashMap<String, IOperand> operand = reader_.getOperandInfo();
		String str = "X,1.1,2.2,3.3:Y,4.4,5.5,6.6:Z,7.7,8.8,9.9";
		HashMap<String, double[]> testPoint = reader_
				.getTestPoint(str, operand);

		assertEquals(1.1, testPoint.get("X")[0]);
		assertEquals(5.5, testPoint.get("Y")[1]);
		assertEquals(9.9, testPoint.get("Z")[2]);
	}

}
