package test;

import main.ISimplify;
import main.MyFileReader;
import main.MyTreeUtil;
import main.NDecimialNumberArray;
import main.zeroAndRelativeError;
import junit.framework.TestCase;

public class NSimplifyTest extends TestCase {

	MyFileReader reader_ = new MyFileReader();

	ISimplify simplify_ = reader_.getSimplify();

	public NSimplifyTest() {

	}

	public void testRuleAndEquivalanceSimplify() {
		MyTreeUtil.setSimplify(simplify_);
		simplify_.setTestPoint(reader_.getTestPoint(
				"X,1,2,3,4,5:Y,2,4,6,8:Z,3,6,9,12,15,18", reader_
						.getOperandInfo()));

		assertEquals("X", simplify_.ruleAndEquivalanceSimplify("X 1 + 1 -"));
		assertEquals(
				"1 X 1 + + X / 1 X + 1 X / + + 1 + X +",
				simplify_
						.ruleAndEquivalanceSimplify("X X / X 1 + + X / 1 X + 1 X / + X X - S + + 1 + X +"));
		assertEquals("E C", simplify_.ruleAndEquivalanceSimplify("E PI S + C"));
		assertEquals(
				"Y",
				simplify_
						.ruleAndEquivalanceSimplify("1 X + PI S * Z Y + * E E / Y + 1 - +"));
		assertEquals("Z", simplify_
				.ruleAndEquivalanceSimplify("1 X Z + X - * Y 1 + 1 - Y - +"));

	}

	public void testRuleSimplify() {
		MyTreeUtil.setSimplify(simplify_);
		assertEquals("X", simplify_.ruleSimplify("X 0 +"));
		assertEquals("0", simplify_.ruleSimplify("0 1 *"));
		assertEquals("1", simplify_.ruleSimplify("X 0 /"));
		assertEquals("X", simplify_.ruleSimplify("X 0 -"));
		assertEquals("0", simplify_.ruleSimplify("0 S"));
		assertEquals("1", simplify_.ruleSimplify("0 C"));
		assertEquals("0", simplify_.ruleSimplify("PI S"));
		assertEquals("0", simplify_.ruleSimplify("1 ln"));
		assertEquals("1", simplify_.ruleSimplify("E ln"));
		assertEquals("0", simplify_.ruleSimplify("1 log"));
		assertEquals("0", simplify_.ruleSimplify("0 T"));
		assertEquals("0", simplify_.ruleSimplify("PI T"));
	}

	public void testEquivalanceSimplify() {
		MyTreeUtil.setSimplify(simplify_);
		simplify_.setTestPoint(reader_.getTestPoint(
				"X,1,2,3,4,5:Y,2,4,6,8:Z,3,6,9,12,15,18", reader_
						.getOperandInfo()));

		assertEquals("1", simplify_.equivalanceSimplify("X X - 1 +"));
		assertEquals("X", simplify_.equivalanceSimplify("X 1 1 + 1 1 + - +"));
		assertEquals("0", simplify_.equivalanceSimplify("X 1 + 1 X + -"));
		assertEquals("Y", simplify_
				.equivalanceSimplify("Y 1 - Y 1 + + 1 1 + /"));
		assertEquals("Z", simplify_.equivalanceSimplify("1 1 + X X - * Z +"));
		assertEquals("1", simplify_.equivalanceSimplify("X X /"));
		assertEquals("1", simplify_.equivalanceSimplify("Y Y /"));
		assertEquals("1", simplify_.equivalanceSimplify("Z Z /"));
		assertEquals("X Y + Z +", simplify_
				.equivalanceSimplify("X 1 + 1 - Y Z * Z / + Z 0 - +"));
		assertEquals("0", simplify_.equivalanceSimplify("PI S"));
	}

	public void testGetFunctionValue() {
		MyTreeUtil.setSimplify(simplify_);
		simplify_.setTestPoint(reader_.getTestPoint("X,1,2,3:Y,4,5,6:Z,7,8,9",
				reader_.getOperandInfo()));
		NDecimialNumberArray nArray = simplify_.getNArray();
		int count = 0;
		do {
			double valueX = simplify_.getTestPoint().get("X")[nArray
					.getNumOfSymbol("X")] + 1;
			assertEquals(valueX, simplify_.getFunctionValue(MyTreeUtil
					.postOrderToTree("X 1 +")), 1E-4);
			count++;
		} while (!nArray.next());
		assertEquals(27, count);
	}

	public void testIsSameValue() {
		assertTrue(simplify_.isSameValue(1.0, 1.0 + 1E-5));
		assertTrue(!simplify_.isSameValue(1.0 - 1E-10, 1.0 + 1E-4));

		zeroAndRelativeError.setRelativeError(1E-3);
		assertTrue(simplify_.isSameValue(1.0, 1.0 + 1E-4));
		assertTrue(!simplify_.isSameValue(1.0 - 1E-10, 1.0 + 1E-3));
	}

	public void testIsZero() {
		assertTrue(simplify_.isZero(1E-10));
		assertTrue(!simplify_.isZero(1E-9));

		zeroAndRelativeError.setZero(1E-10);
		assertTrue(simplify_.isZero(1E-11));
		assertTrue(!simplify_.isZero(1E-10));
	}

}
